#include <Arduino.h>
#include <FastLED.h>

 
// Motor Pins
const int motorRF = 10;
const int motorRR = 9;
const int motorRS = 11;
const int motorLF = 8;
const int motorLR = 7;
const int motorLS = 6;
 
// Sensor Analog Pins
const int sensorA = 2;
const int sensorB = 3;
const int sensorC = 4;
const int sensorD = 5;

// Sensor Analog Pins
const int sensorAPin = 2;
const int sensorBPin = 3;
const int sensorCPin = 4;
const int sensorDPin = 5;
const int sensorEPin = 5;

int currentState = 0;
int previousState = 0;

int currMotorRSpeed = 0;
int currMotorLSpeed = 0;
bool currMotorRDir = 0;
bool currMotorLDir = 0;

int prevMotorRSpeed = 0;
int prevMotorLSpeed = 0;
bool prevMotorRDir = 0;
bool prevMotorLDir = 0;

int maxSpeedL = 100;
int maxSpeedR = 100;
int minSpeedL = 72;
int minSpeedR = 72;

int sensorPins[6] = {A0, A1, A2, A3, A4};
int sensorLocs[6] = {38, 19, 0, -19, -38};
int sensorOffset[6] = {0, 0, 0, 0, 0};
int sensorVals[6] = {0, 0, 0, 0, 0};


void calibrate();

unsigned long prevTimeColor = 0;
const long interval = 10;

int count = 0;

const int ledPin = 4;
CRGB ledArray[6];
uint8_t noiseArray[6];
int rainbowHue = 0;

void setup() {
 
  // Initialize motor pins
  pinMode(motorRF, OUTPUT);
  pinMode(motorRR, OUTPUT);
  pinMode(motorRS, OUTPUT);
  pinMode(motorLF, OUTPUT);
  pinMode(motorLR, OUTPUT);
  pinMode(motorLS, OUTPUT);

  // Initialize sensor pins
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);

  // Initialize LED strip
  FastLED.addLeds<NEOPIXEL, ledPin>(ledArray, 6);
  FastLED.setMaxPowerInVoltsAndMilliamps(5, 1250);
  FastLED.show();
  FastLED.show();
  FastLED.show();

  Serial.begin(9600);
  calibrate();
}

void driveMotors() {

  if ((currMotorRSpeed != prevMotorRSpeed) || (currMotorRDir != prevMotorRDir)) {
    if (currMotorRDir) {
      digitalWrite(motorRF, HIGH);
      digitalWrite(motorRR, LOW);
    }
    else {
      digitalWrite(motorRF, LOW);
      digitalWrite(motorRR, HIGH);
    }
    analogWrite(motorRS, currMotorRSpeed);
  }
  if ((currMotorLSpeed != prevMotorLSpeed) || (currMotorLDir != prevMotorLDir)) {
    if (currMotorLDir) {
      digitalWrite(motorLF, HIGH);
      digitalWrite(motorLR, LOW);
    }
    else {
      digitalWrite(motorLF, LOW);
      digitalWrite(motorLR, HIGH);
    }
    analogWrite(motorLS, currMotorLSpeed);
  }

  prevMotorRSpeed = currMotorRSpeed;
  prevMotorRDir = currMotorRDir;
  prevMotorLSpeed = currMotorLSpeed;
  prevMotorLDir = currMotorLDir;
}

void calibrate() {
  for (int i = 0; i < 5; i++) {
    int sensorSum = 0;
    for (int j = 0; j < 5; j++) {
      sensorSum += analogRead(sensorPins[i]);
      delay(50);
    }
    sensorOffset[i] = (sensorSum/5);
  }
}

int getState() {
  for (int i = 0; i < 5; i++) {
    int sensorIn = analogRead(sensorPins[i]) - sensorOffset[i];
    if (sensorIn < 0) {
      sensorIn = 0;
    }
    sensorVals[i] = sensorIn;
  }

  int sensorE = (sensorVals[0] > 200) ? 1: 0;
  int sensorD = (sensorVals[1] > 200) ? 1: 0;
  int sensorC = (sensorVals[2] > 200) ? 1: 0;
  int sensorB = (sensorVals[3] > 200) ? 1: 0;
  int sensorA = (sensorVals[4] > 200) ? 1: 0;

  int sensorState = (sensorA) +
                (sensorB * 2) +
                (sensorC * 4) +
                (sensorD * 8) + 
                (sensorE * 16);
  
  Serial.println(sensorState);
  return sensorState;
}

void loop() {
  count++;
  if (count == 25) {
    count = 0;
  }

  unsigned long currTime = millis();

  uint8_t sinBeat = beatsin16(30, 0, 6 - 1, 0, 0);
  ledArray[sinBeat] = CRGB(170, 0, 0);
  fadeToBlackBy(ledArray, 6, 3);
  FastLED.show();

  switch (getState()) {
  case 0: // Idle or line break
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    break;
  case 1: // Left hard
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[2] < 200) {
      driveMotors();
      getState();
      delay(10);
    }
    break;
  case 2: // Left soft
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = 0;
    while(sensorVals[2] < 200) {
      driveMotors();
      getState();
      delay(10);
    }
    break;
   case 3: // Left Soft
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    break;
  case 4: // Straight
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    break;   
  case 5: // Left hairpin
      currMotorRDir = true;
      currMotorRSpeed = minSpeedR;
      currMotorLDir = true;
      currMotorLSpeed = minSpeedL;
    while(getState() != 0){
      driveMotors();
      delay(10);
    }
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[2] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    
    break;
  case 7: // Left right angle
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[1] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    break; 
  case 8: // Right soft
    currMotorRDir = true;
    currMotorRSpeed = 0;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[2] < 200) {
      driveMotors();
      getState();
      delay(10);
    }
    break; 
  case 9: // HAIRPIN LEFT
      currMotorRDir = true;
      currMotorRSpeed = minSpeedR;
      currMotorLDir = true;
      currMotorLSpeed = minSpeedL;
    while(getState() != 0){
      driveMotors();
      delay(10);
    }
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[2] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    break;
  case 10: // Close Y approach
    currMotorRDir = true;
    currMotorRSpeed = 0;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    break;
  // CASE 11 to 15 are UNDEF  
  case 15: // Ultra Left
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[1] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    break; 
  case 16: // Right hard
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[2] < 200) {
      driveMotors();
      getState();
      delay(10);
    }
    break;
  case 17: // Far Y approach
    currMotorRDir = true;
    currMotorRSpeed = 0;
    currMotorLDir = false;
    currMotorLSpeed = minSpeedL;
    break;
  case 18: // Right hairpin
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[3] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    break;
  case 20: // Right hairpin
    currMotorRSpeed = 0;
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    while(sensorVals[3] < 200){
      driveMotors();
      getState();
      delay(10);
    }
  break;
  case 24: // Right hard
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    while(sensorVals[3] < 200){
      driveMotors();
      getState();
      delay(10);
    }
  break;
  // CASE 18 to 27 are UNDEF  
  case 28: // Right right angle D
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    while(sensorVals[3] < 200){
      driveMotors();
      getState();
      delay(10);
    }
    break; 
  // CASE 29 to 30 are UNDEF  
  case 30:
    currMotorRDir = false;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    break;
  case 31: // All active
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    break;
  default:
    currMotorRDir = true;
    currMotorRSpeed = minSpeedR;
    currMotorLDir = true;
    currMotorLSpeed = minSpeedL;
    break;
  }

  driveMotors();
  delay(5);
}

